/////////////////////////////////////////////////////////////////////////////
// Name:        CompletionPage.cpp
// Purpose:     Completion page of the wizard
// Author(s):   Atul, Ashesh
// Created:     2008-07-08
// RCS-ID:      $Id: CompletionPage.cpp,v 1.14 2008/08/18 14:18:35 asheshv Exp $
// Copyright:   (c) EnterpriseDB
/////////////////////////////////////////////////////////////////////////////

#include "TuningTool.h"

// wxWindows headers
#include <wx/wx.h>
#include <wx/busyinfo.h>
#include <wx/utils.h>
#include <wx/wizard.h>
#include <wx/textfile.h>
#include <wx/stattext.h>
#include <wx/richtext/richtextctrl.h>
#include <wx/richtext/richtextstyles.h>
#include <wx/progdlg.h>
#include <wx/gbsizer.h>

// Application headers
#include "CompletionPage.h"
#include "WelcomePage.h"

extern const char * Tuning_xpm[];

#define COMPLETIONPAGE_PROGRESSBAR_RANGE (DYNATUNE_CONFIG_LEN * 3 + 10)

#define EDB_MODIFY_NOTICE \
    _("# NOTE: This has been modified by EnterpriseDB's Tuning Wizard on ")

#define PARAM_VALUE_MODIFIED(p, v) \
    _("#       Original Value for \"") + p + _("\" was \"") + v + _("\"")

#define EDB_ADD_NOTICE \
    _("# NOTE: These parameters are added by EnterpriseDB's Tuning Wizard on ")

#define EDB_PARAM_ADD_BEGINS \
    _("# BEGIN")

#define EDB_PARAM_ADD_ENDS \
    _("# END")

#define TW_DATE_FORMAT \
    _("%Y/%m/%d %H:%M:%S")

//To set tab-width to 8 spaces change this value to 200
//currently it has been set to 6 spaces
#define TW_TAB_SIZE 150

const int BTN_REVIEW=1004;

BEGIN_EVENT_TABLE(CompletionPage, wxWizardPageSimple)
    EVT_BUTTON(BTN_REVIEW, CompletionPage::OnReview)
    EVT_WIZARD_PAGE_CHANGING(wxID_ANY, CompletionPage::OnWizardPageChanging)
END_EVENT_TABLE()

CompletionPage::CompletionPage(wxWizard *parent)
    : wxWizardPageSimple(parent)
{
    SetBackgroundColour(*wxWHITE);
    m_parent = (Wizard *)parent;
    m_backedup = false;
    wxBoxSizer *mainSizer = new wxBoxSizer(wxVERTICAL);
    mainSizer->Add(0, 10);
    wxStaticText *st = new wxStaticText(this, wxID_ANY, 
        _("Configuration parameters for your PostgreSQL installation have "\
          "been calculated based on your desired usage and hardware "\
          "configuration, and merged with your existing configuration file.\n"\
          "\nPlease click the 'Review' button to examine the new "\
          "configuration file"));
    st->Wrap(400);
    mainSizer->Add(st, 0, wxALL, 5);
#ifdef WIN32
    mainSizer->Add(0, 20);
#else
    mainSizer->Add(0, 5);
#endif
    m_review = new wxButton(this, BTN_REVIEW, _("Review"));
    mainSizer->Add(m_review, 0, wxALL | wxALIGN_CENTER, 5);

    wxDateTime now = wxDateTime::Now();
    wxString date = now.Format(_("%Y%m%d%H%M%S"));

    m_backupFileName.Printf(_("postgresql.conf."));
    m_backupFileName.Append(date.c_str());
#ifdef WIN32
    mainSizer->Add(0, 50);
#else
    mainSizer->Add(0,5);
#endif //WIN32
    st = new wxStaticText(this, wxID_ANY, 
        _("Upon clicking the 'Next' button, the configuration file wlll be "\
          "saved and the original file renamed to: "));
    st->Wrap(300);
    mainSizer->Add(st, 0, wxALL | wxALIGN_LEFT, 5);

    st = new wxStaticText(this, wxID_ANY, m_backupFileName
#ifdef WIN32
                         + _("                                "));
#else
                         );
#endif
    wxFont boldDflFont = st->GetFont();
    boldDflFont.SetWeight(wxFONTWEIGHT_BOLD);
    st->SetFont(boldDflFont);
    st->Wrap(300);
    mainSizer->Add(st, 0, wxALL | wxALIGN_LEFT, 5);
    
    st = new wxStaticText(this, wxID_ANY, 
        wxT("\nIf you do not wish to use the new configuration file, "\
          "please press 'Back' to change the options you have selected, "\
          "or 'Cancel' to exit."));
    st->Wrap(300);
    mainSizer->Add(st, 0, wxALL | wxALIGN_LEFT, 5);
    mainSizer->AddStretchSpacer();
    wxGridBagSizer *imageSizer = new wxGridBagSizer();
    imageSizer->SetMinSize(15, 5);
#if wxUSE_STATBMP
    wxBoxSizer *bmpSizer = new wxBoxSizer(wxVERTICAL);
    imageSizer->Add(
        bmpSizer, wxGBPosition(0,32), wxGBSpan(3,1), wxALIGN_RIGHT
        );
    // Create and run the wizard
    wxBitmap bitmap = wxBitmap(Tuning_xpm);
    if ( bitmap.Ok() )
    {
        wxStaticBitmap *statbmp = new wxStaticBitmap(this, wxID_ANY, bitmap);
        bmpSizer->Add(statbmp, 0, wxALL, 0);
    }
#endif
    mainSizer->Add(imageSizer);
    SetSizer(mainSizer);
    mainSizer->Fit(this);
    m_progressDlg = NULL;
}

void CompletionPage::OnWizardPageChanging(wxWizardEvent& event)
{
    if (event.GetDirection())
    {
        // Going forward (next button click)
        wxString strOutFileName = BackupConfFile();
	UpdateConfFile(strOutFileName);
	// destroy the cancel button. 
	// The back button should also be destroyed. But doing so causes 
	// a segmentation fault
	m_parent->FindWindow(wxID_CANCEL)->Destroy();
    }
}

void CompletionPage::OnReview(wxCommandEvent& WXUNUSED(event))
{
    int svr_utilization = -1, workload_profile = -1;
    Server *selectedServer =
        m_parent->GetServerConfiguration(svr_utilization,
                                         workload_profile);
    wxDateTime now = wxDateTime::Now();
    wxDialog textPadDlg;
    ConfigLine* aLine = NULL, *prevLine = NULL;
    textPadDlg.Create(this, -1, _("postgresql.conf"),
                      wxDefaultPosition, wxSize(700, 600),
                      wxCAPTION | wxSYSTEM_MENU | wxCLOSE_BOX | 
                      wxRESIZE_BORDER | wxMAXIMIZE_BOX);
    InitProgressBar(true);
    aLine = GetPostgreSQLConfiguration();
    UpdateProgressBar(_("Preparing to show the configuration file"));
    wxRichTextCtrl textpad(&textPadDlg, -1, wxEmptyString,
        wxDefaultPosition, textPadDlg.GetVirtualSize(),
        wxVSCROLL | wxHSCROLL | wxNO_BORDER | wxWANTS_CHARS |
        wxRE_READONLY | wxRE_MULTILINE);
    textpad.BeginSuppressUndo();
    wxFont lucidaFont(
             10, wxFONTFAMILY_DEFAULT, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL,
             false, wxT("Lucida Console"), wxFONTENCODING_SYSTEM);
    textpad.SetFont(lucidaFont);
    UpdateProgressBar(wxT("Loading configuration file ..."));
    bool dontAddNewLine = true;

    wxArrayInt tabs;
    int cTabVal = TW_TAB_SIZE;
    for(int i=1; i < 10; i++, cTabVal += TW_TAB_SIZE)
        tabs.Add(cTabVal);
    wxRichTextAttr backgroundColourAttr;
    backgroundColourAttr.SetTabs(tabs);
    wxRichTextAttr normalColorAttr;
    normalColorAttr.SetTabs(tabs);
    wxColor textColor(20, 200, 100), backColor(255, 244, 220), redcolor(255, 0, 0);
    textpad.BeginStyle(normalColorAttr);
    while(true)
    {
        prevLine = aLine;
        if(aLine->m_isModified)
        {
            UpdateProgressBar(_("Loading configuration file ..."));
            backgroundColourAttr.SetBackgroundColour(backColor);
            if(!aLine->m_original_value.IsEmpty())
            {
                backgroundColourAttr.SetTextColour(textColor);
                textpad.BeginStyle(backgroundColourAttr);
                textpad.WriteText(EDB_MODIFY_NOTICE + now.Format(TW_DATE_FORMAT));
                textpad.Newline();
                textpad.WriteText(PARAM_VALUE_MODIFIED(
                    (aLine->m_parameter), (aLine->m_original_value)));
                textpad.Newline();
                textpad.EndStyle();
                backgroundColourAttr.SetTextColour(redcolor);
                textpad.BeginStyle(backgroundColourAttr);
                textpad.WriteText(aLine->m_aLine);
            }
            else if(aLine->m_aLine.StartsWith(EDB_ADD_NOTICE) ||
                aLine->m_aLine.IsSameAs(EDB_PARAM_ADD_BEGINS) ||
                aLine->m_aLine.IsSameAs(EDB_PARAM_ADD_ENDS))
            {
                backgroundColourAttr.SetTextColour(wxColour(20, 200, 100));
                textpad.BeginStyle(backgroundColourAttr);
                textpad.WriteText(aLine->m_aLine);
            }
            else
            {
                backgroundColourAttr.SetTextColour(wxColour(255, 0, 0));
                textpad.BeginStyle(backgroundColourAttr);
                textpad.WriteText(aLine->m_aLine);
            }
            textpad.EndStyle();
            textpad.BeginStyle(normalColorAttr);
        }
        else
        {
            textpad.WriteText(aLine->m_aLine);
        }
        aLine = aLine->m_next;
        delete prevLine;
        if(aLine == NULL)
            break;
        textpad.Newline();
    }

    InitProgressBar(false);

    textpad.SetEditable(false);
    textpad.AdjustScrollbars();
    textPadDlg.ShowModal();
}


wxString CompletionPage::BackupConfFile()
{
    wxString inFileName, outFileName;
    int svr_utilization = -1, workload_profile = -1;

    Server *selectedServer = m_parent->GetServerConfiguration(svr_utilization, workload_profile);
    WelcomePage *wp = (WelcomePage *)(m_parent->GetFirstPage());
    if (wp->IsInstallationDetected())
    {
        inFileName.Printf(selectedServer->m_dataDirectory.c_str());
        inFileName.Append(_("/postgresql.conf"));
        outFileName.Printf(selectedServer->m_dataDirectory.c_str());
        outFileName.Append(_("/") + m_backupFileName);
    }
    else
    {
        inFileName.Printf(wp->GetPath().c_str());
        outFileName.Printf(wp->GetDirectory().c_str());
        outFileName.Append(_("/"));
        outFileName.Append(m_backupFileName);
    }
    if (!m_backedup  && wxFile::Exists(inFileName))
    {
        wxTextFile infile(inFileName);
        wxTextFile outfile(outFileName);
        infile.Open();
        outfile.Create();
        wxString buffer;
        for (buffer = infile.GetFirstLine(); !infile.Eof();
             buffer = infile.GetNextLine())
        outfile.AddLine(buffer);
        infile.Close();
        outfile.Write();
        outfile.Close();
    }
    m_backedup = true;
    return outFileName;
}

void CompletionPage::UpdateConfFile(wxString backupFileName)
{
    wxString outFileName;
    int svr_utilization = -1;
    int workload_profile = -1;
    Server* server = m_parent->GetServerConfiguration(svr_utilization, workload_profile);
    WelcomePage *wp = (WelcomePage *)(m_parent->GetFirstPage());
    if (wp->IsInstallationDetected())
    {
        outFileName.Printf(server->m_dataDirectory.c_str());
        outFileName.Append(_("/postgresql.conf"));
    }
    else
    {
        outFileName.Printf(wp->GetPath().c_str());
    }
    ConfigLine* aLine = GetPostgreSQLConfiguration();
    if(wxFile::Exists(outFileName))
    {
        if(!wxFile::Access(outFileName.wx_str(), wxFile::write))
        {
            wxString strFileAccessible = _("File Not Accessible: ");
            strFileAccessible.append(outFileName);
            strFileAccessible.append(
                _("\nMake sure that you are running this application as an "\
                  "administrator."));
            wxLogError(strFileAccessible);
            return;
        }
    }
    ConfigLine* prevLine = NULL;
    wxTextFile fOutFile(outFileName);
    fOutFile.Create();
    wxDateTime now = wxDateTime::Now();
    while(aLine)
    {
        prevLine = aLine;
        if(aLine->m_isModified 
            && !aLine->m_original_value.IsEmpty())
        {
            fOutFile.AddLine(EDB_MODIFY_NOTICE + now.Format(TW_DATE_FORMAT));
            fOutFile.AddLine(PARAM_VALUE_MODIFIED(
                (aLine->m_parameter), (aLine->m_original_value)));
        }
        fOutFile.AddLine(aLine->m_aLine);
        aLine = aLine->m_next;
        delete prevLine;
    }
    fOutFile.Write();
    fOutFile.Close();
}

ConfigLine::ConfigLine()
{
    m_next = NULL;
    m_isModified = false;
}

void ConfigLine::GenerateNext(ConfigLine*& current, ConfigLine*& prev)
{
    if(!current)
        return;
    prev = current;
    current = new ConfigLine();
    prev->m_next = current;
}

void CompletionPage::InitProgressBar(bool show)
{
    if(show && !m_progressDlg)
    {
        m_progressDlg = new wxProgressDialog(
            _("EnterpriseDB's Tuning Wizard"),
            _("**************************************************************"),
            COMPLETIONPAGE_PROGRESSBAR_RANGE, this, 
            wxPD_APP_MODAL | wxPD_AUTO_HIDE | wxPD_SMOOTH);
        m_progressDlg->SetVirtualSize(200, 50);
        m_progressDlg->Update(0, wxEmptyString);
        m_current_progress_value = 0;
    }
    else if(m_progressDlg)
    {
        m_progressDlg->Destroy();
        m_progressDlg = NULL;
    }
}

void CompletionPage::UpdateProgressBar(wxString status, int increaseBy)
{
    if(!m_progressDlg)
        return;
    if(m_current_progress_value > COMPLETIONPAGE_PROGRESSBAR_RANGE)
        m_current_progress_value = COMPLETIONPAGE_PROGRESSBAR_RANGE;
    m_current_progress_value += increaseBy;
    m_progressDlg->Update(m_current_progress_value, status);
}


ConfigLine* CompletionPage::GetPostgreSQLConfiguration()
{
    wxString inFileName;
    int svr_utilization = -1, workload_profile = -1;
    ConfigLine *result = new ConfigLine();
    ConfigLine *currentLine = result;
    ConfigLine *prevLine = NULL;
    wxDateTime now = wxDateTime::Now();

    UpdateProgressBar(_("Initialize Parser"));
    Server *selectedServer = m_parent->GetServerConfiguration(svr_utilization,
                                                            workload_profile);
    WelcomePage *wp = (WelcomePage *)(m_parent->GetFirstPage());
    if (wp->IsInstallationDetected())
    {
        inFileName.Printf(selectedServer->m_dataDirectory.c_str());
        if(!m_backedup)
            inFileName.Append(_("/postgresql.conf"));
        else
            inFileName.Append(_("/") + m_backupFileName);
    }
    else
    {
        if(!m_backedup)
            inFileName.Printf(wp->GetPath().c_str());
        else
        {
            inFileName.Append(_("/") + m_backupFileName);
            inFileName.Printf(wp->GetDirectory().c_str());
        }
    }
    UpdateProgressBar(_("Reading ") + inFileName + _(" ..."));
    selectedServer->ClearConfigFlags();
    UpdateProgressBar(_("Starting updating configuration file ..."));
    int no_updated_parameters = 0;
    wxString buffer;
    
    wxTextFile infile(inFileName);
    if(infile.Exists())
    {
        infile.Open();
        wxString matched_parameter, original_value, new_value;
        int i = 0;
	for ( buffer = infile.GetFirstLine(); !infile.Eof();
              buffer = infile.GetNextLine() )
	{
            wxString new_buffer
                = selectedServer->UpdateConfigBuffer(buffer, 
                                                        matched_parameter,
                                                        original_value,
                                                        new_value);
	    if(!matched_parameter.IsEmpty())
	    {
                if(original_value != new_value)
		{
                    UpdateProgressBar(_("Updating parameter ") +
                                      matched_parameter + _(" ..."));
                    currentLine->m_parameter = matched_parameter;
                    currentLine->m_original_value = original_value;
                    currentLine->m_isModified = true;
                    buffer = new_buffer;
                }
                no_updated_parameters++;
            }
            currentLine->m_aLine = buffer;
            ConfigLine::GenerateNext(currentLine, prevLine);
	}
    }
    if(no_updated_parameters < DYNATUNE_CONFIG_LEN - 1)
    {
        //Add Notice
        currentLine->m_aLine = EDB_ADD_NOTICE + now.Format(TW_DATE_FORMAT);
        currentLine->m_isModified = true;
        ConfigLine::GenerateNext(currentLine, prevLine);
        //Add begin
        currentLine->m_aLine = EDB_PARAM_ADD_BEGINS;
        currentLine->m_isModified = true;
        ConfigLine::GenerateNext(currentLine, prevLine);
        while(no_updated_parameters < DYNATUNE_CONFIG_LEN - 1)
        {
            buffer = selectedServer->GetNextNonUpdatedConfiguration();
            if(!buffer.IsEmpty())
            {
                currentLine->m_aLine = buffer;
                currentLine->m_isModified = true;
                ConfigLine::GenerateNext(currentLine, prevLine);
            }
            no_updated_parameters++;
        }
        currentLine->m_aLine = EDB_PARAM_ADD_ENDS;
        currentLine->m_isModified = true;
        ConfigLine::GenerateNext(currentLine, prevLine);
    }

    UpdateProgressBar(_("Closing configuration file ..."));
    infile.Close();

    if(prevLine)
        prevLine->m_next = NULL;
    else
        result = NULL;
    delete(currentLine);
    return result;
}

