/////////////////////////////////////////////////////////////////////////////
// Name:        FileSelectorCombo.cpp
// Purpose:     File Selector combobox implementation
// Author(s):   Atul, Ashesh
// Created:     2008-07-08
// RCS-ID:      $Id: FileSelectorCombo.cpp,v 1.2 2008/08/21 06:43:12 atuld Exp $
// Copyright:   (c) EnterpriseDB
/////////////////////////////////////////////////////////////////////////////

// wxWindows headers
#include <wx/wx.h>
#include <wx/busyinfo.h>
#include <wx/utils.h>
#include <wx/wizard.h>

#ifdef __WXMSW__
#include <wx/msw/registry.h>
#endif

// Application headers
#include "FileSelectorCombo.h"


FileSelectorCombo::FileSelectorCombo(wxWindow *parent,
									 wxWindowID id,
									 const wxString& value,
									 const wxPoint& pos,
									 const wxSize& size,
									 long style,
									 const wxValidator& validator,
									 const wxString& name)
									 : wxComboCtrl()
{
	Init();
	Create(parent, id, value,
		pos, size,
		// Style flag wxCC_STD_BUTTON makes the button
		// behave more like a standard push button.
		style | wxCC_STD_BUTTON,
		validator, name);

	//
	// Prepare custom button bitmap (just '...' text)
	wxMemoryDC dc;
	wxBitmap bmp(12,16);
	dc.SelectObject(bmp);

	// Draw transparent background
	wxColour magic(255,0,255);
	wxBrush magicBrush(magic);
	dc.SetBrush( magicBrush );
	dc.SetPen( *wxTRANSPARENT_PEN );
	dc.DrawRectangle(0,0,bmp.GetWidth(),bmp.GetHeight());

	// Draw text
	wxString str = wxT("...");
	int w,h;
	dc.GetTextExtent(str, &w, &h, 0, 0);
	dc.DrawText(str, (bmp.GetWidth()-w)/2, (bmp.GetHeight()-h)/2);

	dc.SelectObject( wxNullBitmap );

	// Finalize transparency with a mask
	wxMask *mask = new wxMask( bmp, magic );
	bmp.SetMask( mask );

	SetButtonBitmaps(bmp,true);
}

void FileSelectorCombo::OnButtonClick()
{
	// Show standard wxFileDialog on button click
	m_filedlg = new wxFileDialog(this,
		wxT("Choose File"),
		wxEmptyString,
		GetValue(),
		wxT("Conf files (*.conf)|*.conf"),
		wxFD_OPEN);

	if ( m_filedlg->ShowModal() == wxID_OK )
	{
		SetValue(m_filedlg->GetPath());
		m_filename = m_filedlg->GetFilename();
		m_directory = m_filedlg->GetDirectory();
		m_path = m_filedlg->GetPath();
	}
}

