/////////////////////////////////////////////////////////////////////////////
// Name:        TuningTool.cpp
// Purpose:     EnterpriseDB Tuning Wizard
// Author(s):   Atul, Ashesh
// Created:     2008-07-08
// RCS-ID:      $Id: TuningTool.cpp,v 1.11 2008/08/18 14:18:35 asheshv Exp $
// Copyright:   (c) EnterpriseDB
/////////////////////////////////////////////////////////////////////////////

#include "TuningTool.h"

// wxWindows headers
#include <wx/wx.h>
#include <wx/app.h>
#include <wx/cmdline.h>
#include <wx/dir.h>

// Application headers
#include "Wizard.h"
#include "ThankYouPage.h"
#include "WelcomePage.h"
#include "Server.h"
#include "images/TuningWizard.xpm"

IMPLEMENT_APP(TuningTool)

BEGIN_EVENT_TABLE(TuningTool, wxApp)
    EVT_WIZARD_CANCEL(wxID_ANY, TuningTool::OnWizardCancelled)
    EVT_WIZARD_FINISHED(wxID_ANY, TuningTool::OnWizardFinished)
END_EVENT_TABLE()

// The Application!
bool TuningTool::OnInit()
{
    wxString mirrorListUrl;
    wxString applicationListUrl;
    wxString language;

    SetAppName(wxT("TuningWizard"));
    SetVendorName(wxT("EnterpriseDB"));

    // Command line options
    static const wxCmdLineEntryDesc cmdLineDesc[] = 
    {
        {wxCMD_LINE_SWITCH, wxT("h"), wxT("help"), wxT("show this help message"),
         wxCMD_LINE_VAL_NONE, wxCMD_LINE_OPTION_HELP},
        {wxCMD_LINE_OPTION, wxT("l"), wxT("language"),
         wxT("use the specified language in the UI"), wxCMD_LINE_VAL_STRING},
        {wxCMD_LINE_NONE}
    };

    wxCmdLineParser cmdParser(cmdLineDesc, argc, argv);
    if (cmdParser.Parse() != 0) 
        return false;

    if (!cmdParser.Found(wxT("l"), &language))
        language = wxEmptyString;

    // Hack for the PostgreSQL installer - it might ask for the default language
    if (language == wxT("DEFAULT"))
        language = wxEmptyString;

    // Initialize our locale and load the language catalog...
    // InitializeLocale(argv[0], language);

    // Create and run the wizard
    wxBitmap bitmap = wxBitmap(TuningWizard_xpm);
    wizard = new Wizard(NULL, bitmap);
    bool retval = wizard->RunWizard(wizard->GetFirstPage()); 

    return retval;
}

void TuningTool::OnWizardCancelled(wxWizardEvent &evt)
{
    if (wxMessageBox(
            wxT("Are you sure you want to close the EnterpriseDB Tuning Wizard?"),
            wxT("Exit wizard?"), wxICON_QUESTION | wxYES_NO) == wxNO)
    {
        evt.Veto();
        return;
    }
    wizard->Destroy();
}

void TuningTool::OnWizardFinished(wxWizardEvent &evt)
{
    if (wizard->IsAdditionalInfoChecked())
        wxLaunchDefaultBrowser(
              wxT("http://www.enterprisedb.com/tservices/overview.do"));
    wizard->Destroy();
}

void TuningTool::InitializeLocale(wxChar *argv0, const wxString &lang)
{
    wxString appPath = wxPathOnly(argv0);
    wxString i18nPath;

    if( appPath.IsEmpty())
        appPath = wxT(".");

    if(wxDir::Exists(appPath + wxT("/i18n")))
        i18nPath = appPath + wxT("/i18n");
    else if(wxDir::Exists(appPath + wxT("/../TuningWizard/i18n")))
        i18nPath = appPath + wxT("/../TuningWizard/i18n");
    else if(wxDir::Exists(appPath + wxT("/../i18n")))
        i18nPath = appPath + wxT("/../i18n");

    wxLocale *locale = new wxLocale();
    locale->AddCatalogLookupPathPrefix(i18nPath);

    const wxLanguageInfo *langInfo;

    if (!lang.IsEmpty())
        langInfo = wxLocale::FindLanguageInfo(lang);
    else
        langInfo = wxLocale::GetLanguageInfo(wxLANGUAGE_DEFAULT);

    if(locale->Init(langInfo->Language), wxLOCALE_LOAD_DEFAULT)
        locale->AddCatalog(wxT("TuningWizard"));
}
