/////////////////////////////////////////////////////////////////////////////
// Name:        WsUtils.cpp
// Purpose:     Utility class for working with dynatune webservice
// Author(s):   Atul, Ashesh
// Created:     2008-07-08
// RCS-ID:      $Id: WSUtils.cpp,v 1.7 2008/08/18 14:18:35 asheshv Exp $
// Copyright:   (c) EnterpriseDB
/////////////////////////////////////////////////////////////////////////////

#include <wx/wx.h>
#include <wx/string.h>
#include <wx/log.h>

#include "WSUtils.h"
#include "AuthenticationServiceSoapBinding.nsmap"

#ifndef WIN32
#include <unistd.h>
#include <locale.h>
#include <sys/sysinfo.h>
#include <sys/utsname.h>
#include <uuid/uuid.h>
#endif /* WIN32 */

#ifdef ___CREATE_LOG_
#include <wx/textfile.h>
#endif

#define TW_INST_TYPE "tune"
#define TW_FEATURES "tune"
#define TW_INST_TUNING "true"
#define TW_UPDATE_NTF  "false"

int WSUtils::IsAllHex(const char* const must_be_hex) {
	char *copy_of_param = (char *)malloc((strlen(must_be_hex))+1);
	int result;
	result = strtok (strcpy(copy_of_param, must_be_hex), "0123456789ABCDEFabcdef-") == 0;
	free(copy_of_param);
	return result;
}

char *WSUtils::ConvertToHexString(const char *str) {
	char *newstr;
    const char *cpold;
    char *cpnew;

	if(str == NULL) {
		fprintf(stderr, "StrToHexStr: NULL Input String\n");
		return NULL;
	}

	newstr = (char *)malloc((strlen(str)*2)+1);
	cpold = str;
	cpnew = newstr;

    while('\0' != *cpold) {
            sprintf(cpnew, "%02X", (unsigned char)(*cpold++));
            cpnew+=2;
    }
    *(cpnew) = '\0';

    return(newstr);
}

char WSUtils::HexCharToChar(char c)
{
    int value = c - '0';
    if(value < 10)
        return value;
    value = c - 'A';
    if(value < 6)
        return value + 10;
    value = c - 'a';
    if(value < 6)
        return value + 10;
    return 0;
}

char *WSUtils::HexStrToStr(const char *str) {
	int stringLen; //this must be even value
	char *newstr;
	int i, j;
	
	if(str == NULL) {
		fprintf(stderr, "Invalid Hex String:%s\n", str);
		return NULL;
	}

	stringLen=(int)strlen(str);
	
	//check if its not odd and valid characters
	if(((stringLen & 0x1) != 0) && !IsAllHex(str)) {
		fprintf(stderr, "Invalid Hex String:%s\n", str);
		return NULL;
	}

	newstr = (char *)malloc((stringLen/2)+1);

	
	for(i=0, j=0; i<stringLen; i+=2, j++) {
        newstr[j] = (((int)HexCharToChar(str[i])) * 16) + ((int)HexCharToChar(str[i+1]));
	}
    newstr[stringLen/2] = '\0';
    return(newstr);
}

char* WSUtils::GetLocaleName() {
#ifdef WIN32
    LCID  lcid=GetSystemDefaultLCID();
    char hexCLID[64];
    char lszValue[255];
    
    HKEY hKey;
    LONG returnStatus;
    DWORD dwType=REG_SZ;
    DWORD dwSize=255;

    char *language=(char *)malloc(sizeof(char)*10);
    int n;

    //convert it to 4 digit hex value
    sprintf(hexCLID, "%04x", lcid);
   
    //obtain it from registry
    returnStatus
          = RegOpenKeyExA(HKEY_LOCAL_MACHINE, "SOFTWARE\\Classes\\MIME\\Database\\Rfc1766", 0L,  KEY_ALL_ACCESS, &hKey);
    if (returnStatus == ERROR_SUCCESS)
    {
         returnStatus = RegQueryValueExA(hKey, hexCLID, NULL, &dwType, (LPBYTE)&lszValue, &dwSize);
         if (returnStatus == ERROR_SUCCESS)
         {
             //parse at first ;
             sscanf(lszValue, "%9[^;]%n", language, &n);
         }
         else
         {
             sprintf(language, "Unknown");
         }
    }
    else
    {
        sprintf(language, "Unknown");
    }
    RegCloseKey(hKey);
    return language;
#else
    char *language=(char *)malloc(sizeof(char)*15);
    strncpy(language, setlocale(LC_CTYPE, NULL), 14);
    return language;
#endif /* WIN32 */
}

char *WSUtils::GetTotalMemoryInGB() {
    double DIV = 1073741824.0;
    double ans = 0.0;
    char *cAns = NULL;
#ifdef WIN32
    MEMORYSTATUSEX statex;
    statex.dwLength = sizeof (statex);
    cAns = (char *)malloc(sizeof(char)*11);
    GlobalMemoryStatusEx (&statex);
    ans = statex.ullTotalPhys/DIV;
    _snprintf(cAns, 10, "%.4f", ans);
#else
    struct sysinfo sinfo;
    cAns = (char *)malloc(sizeof(char)*11);
    if(sysinfo(&sinfo) == 0)
    {
    	ans = (sinfo.totalram * sinfo.mem_unit )/ DIV;
    }
    snprintf(cAns, 10, "%.4f", ans);
#endif /* WIN32 */
    return cAns;
}

char *WSUtils::GetTotalMemoryInMB() {
    double DIV = 1048576;
    long ans = 0;
    char *cAns = NULL;
#ifdef WIN32
    MEMORYSTATUSEX statex;
    statex.dwLength = sizeof (statex);
    cAns = (char *)malloc(sizeof(char)*11);
    statex.dwLength = sizeof (statex);
    GlobalMemoryStatusEx (&statex);
    ans = (long)(statex.ullTotalPhys/DIV);
    _snprintf(cAns, 10, "%d", ans);
#else
    struct sysinfo sinfo;
    cAns = (char *)malloc(sizeof(char)*11);
    if(sysinfo(&sinfo) == 0)
    {
    	ans = (long)(sinfo.totalram * sinfo.mem_unit/ DIV);
    }
    snprintf(cAns, 10, "%d", ans);
#endif /* WIN32 */
    return cAns;
}

#ifdef WIN32
RPC_CSTR g_strUUID = NULL;
#else
char *g_strUUID = NULL;
#endif

char* WSUtils::Genguid() {
    if(g_strUUID)
        return (char *)g_strUUID;
#ifdef WIN32
    UUID uuid;
    // create UUID
    UuidCreate(&uuid);
    // create ansi string like "XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX"
    UuidToStringA(&uuid, &g_strUUID);
    return (char *)g_strUUID;
#else
#ifndef UUID_UNPARSED_SIZE
#define UUID_UNPARSED_SIZE 512
#endif
    uuid_t uuid;
    g_strUUID = (char *)malloc((UUID_UNPARSED_SIZE + 1 )* sizeof(char));
    uuid_generate(uuid);
    uuid_unparse(uuid, g_strUUID);
    return g_strUUID;
#endif /* WIN32 */
}

void WSUtils::Releaseguid() {
    // clean up
    if(g_strUUID)
    {
#ifdef WIN32
        RpcStringFreeA(&g_strUUID);
#else 
        free(g_strUUID);
#endif /* WIN32 */
        g_strUUID = NULL;
    }
}

int WSUtils::validateUser(AuthenticationServiceSoapBindingProxy& authBody,
                          wxString &uuid, wxString &db_version,
                          wxString &userid, wxString &passwd,
                          wxString &server_util, wxString &work_prof)
{

    int result = 0;


    char l_os[64] = {0};//determine via environment variable
    char l_processor_no[10] = {0};//determine via environment variable
    char l_processor_arch[64] =  {0};//determine via environment variable
    char l_processor_type[256] = {0};////determine via environment variable
    char *l_language = WSUtils::GetLocaleName();
    char *l_ram_gb = WSUtils::GetTotalMemoryInGB();
    char *l_ram_mb = WSUtils::GetTotalMemoryInMB();
#ifdef WIN32
    GetEnvironmentVariable("OS", l_os, sizeof(l_os));
    GetEnvironmentVariable("NUMBER_OF_PROCESSORS", l_processor_no, sizeof(l_processor_no));
    GetEnvironmentVariable("PROCESSOR_ARCHITECTURE", l_processor_arch, sizeof(l_processor_arch));
    GetEnvironmentVariable("PROCESSOR_IDENTIFIER", l_processor_type, sizeof(l_processor_type));
#else    
    struct utsname uts;
    int i_no_processors = 1;
    
    uname(&uts);
    snprintf(l_os, 63, "%s %s", uts.sysname, uts.release);
    
#ifdef _SC_NPROCESSORS_ONLN
    i_no_processors = sysconf(_SC_NPROCESSORS_ONLN);
#endif
    snprintf(l_processor_no, 10, "%d", i_no_processors);
    snprintf(l_processor_arch, 63, "%s", uts.machine);
    snprintf(l_processor_type, 63, "%s", uts.machine);
#endif /* WIN32 */

    char *hex_uuid
             = WSUtils::ConvertToHexString(
                              (const char *)(uuid.mb_str(wxConvUTF8)));
    char *hex_installer_type = WSUtils::ConvertToHexString(TW_INST_TYPE);
    char *hex_language       = WSUtils::ConvertToHexString(l_language);
    char *hex_features       = WSUtils::ConvertToHexString(TW_FEATURES);
    char *hex_install_tuning = WSUtils::ConvertToHexString(TW_INST_TUNING);
    char *hex_update_notify  = WSUtils::ConvertToHexString(TW_UPDATE_NTF);
    char *hex_db_version
             = WSUtils::ConvertToHexString(
                               (const char *)db_version.mb_str(wxConvUTF8));
    char *hex_os             = WSUtils::ConvertToHexString(l_os);
    char *hex_no_of_proc     = WSUtils::ConvertToHexString(l_processor_no);
    char *hex_proc_arch      = WSUtils::ConvertToHexString(l_processor_arch);
    char *hex_chk_cnt        = WSUtils::ConvertToHexString("1");
    char *hex_proc_type      = WSUtils::ConvertToHexString(l_processor_type);
    char *hex_ram_gb         = WSUtils::ConvertToHexString(l_ram_gb);
    char *hex_disk_gb        = WSUtils::ConvertToHexString("0");
    char *hex_shared_mem_mb  = WSUtils::ConvertToHexString(l_ram_mb);
    char *hex_userid
             = WSUtils::ConvertToHexString(
                               (const char *)userid.mb_str(wxConvUTF8));
    char *hex_passwd
             = WSUtils::ConvertToHexString(
                               (const char *)passwd.mb_str(wxConvUTF8));
    char *hex_server_util
             = WSUtils::ConvertToHexString(
                               (const char *)server_util.mb_str(wxConvUTF8));
    char *hex_work_profile
             = WSUtils::ConvertToHexString(
                               (const char *)work_prof.mb_str(wxConvUTF8));
    
#ifdef ___CREATE_LOG_
    wxTextFile aLog(wxT("edb_validateUser.log"));
    aLog.Create();
    {
        wxString wStrBuf;
        wStrBuf = wxString::Format(_("%s:%s:"), "UUID", hex_uuid) + uuid;
        aLog.AddLine(wStrBuf);
        wStrBuf
            = wxString::Format(
                               _("%s:%s:%s"), "installer_type",
                               hex_installer_type, TW_INST_TYPE);
        aLog.AddLine(wStrBuf);
        wStrBuf
            = wxString::Format(
                               _("%s:%s:%s"), "language",
                               hex_language, l_language);
        aLog.AddLine(wStrBuf);
        wStrBuf
            = wxString::Format(
                               _("%s:%s:%s"), "features",
                               hex_features, TW_FEATURES);
        aLog.AddLine(wStrBuf);
        wStrBuf
           = wxString::Format(_("%s:%s:%s"), "install_tuning",
                              hex_install_tuning, TW_INST_TUNING);
        aLog.AddLine(wStrBuf);
        wStrBuf
           = wxString::Format(_("%s:%s:%s"), "update_notify",
                              hex_update_notify, TW_UPDATE_NTF);
        aLog.AddLine(wStrBuf);
        wStrBuf
            = wxString::Format(_("%s:%s:"), "db_version", hex_db_version) +
              db_version;
        aLog.AddLine(wStrBuf);
        wStrBuf = wxString::Format(_("%s:%s:%s"), "os", hex_os, l_os);
        aLog.AddLine(wStrBuf);
        wStrBuf
            = wxString::Format(
                               _("%s:%s:%s"), "no_of_proc",
                               hex_no_of_proc, l_processor_no);
        aLog.AddLine(wStrBuf);
        wStrBuf
            = wxString::Format(
                               _("%s:%s:%s"), "proc_arch",
                               hex_proc_arch, l_processor_arch);
        aLog.AddLine(wStrBuf);
        wStrBuf = wxString::Format(_("%s:%s:%s"), "chk_cnt", hex_chk_cnt, "1");
        aLog.AddLine(wStrBuf);
        wStrBuf
            = wxString::Format(
                               _("%s:%s:%s"), "proc_type",
                               hex_proc_type, l_processor_type);
        aLog.AddLine(wStrBuf);
        wStrBuf
            = wxString::Format(_("%s:%s:%s"), "ram_gb", hex_ram_gb, l_ram_gb);
        aLog.AddLine(wStrBuf);
        wStrBuf = wxString::Format(_("%s:%s:%s"), "disk_gb", hex_disk_gb, "0");
        aLog.AddLine(wStrBuf);
        wStrBuf
            = wxString::Format(
                               _("%s:%s:%s"), "shared_mem_mb",
                               hex_shared_mem_mb, l_ram_mb);
        aLog.AddLine(wStrBuf);
        wStrBuf = wxString::Format(_("%s:%s:"), "userid", hex_userid) + userid;
        aLog.AddLine(wStrBuf);
        wStrBuf = wxString::Format(_("%s:%s:"), "passwd", hex_passwd) + passwd;
        aLog.AddLine(wStrBuf);
        wStrBuf
            = wxString::Format(_("%s:%s:"), "server_util", hex_server_util) +
              server_util;
        aLog.AddLine(wStrBuf);
        wStrBuf
            = wxString::Format(_("%s:%s:"), "work_profile", hex_work_profile) +
              work_prof;
    }
    aLog.Write();
    aLog.Close();
#endif //___CREATE_LOG_

    //char *createNewUserParams[]  = {hex_uuid, hex_installer_type,
    //    hex_language, hex_features, hex_install_tuning, hex_update_notify,
    //    hex_db_version, hex_os, hex_no_of_proc, hex_proc_arch, hex_chk_cnt,
    //    hex_proc_type, hex_ram_gb, hex_disk_gb, hex_shared_mem_mb, hex_userid,
    //    hex_server_util, hex_work_profile};

    char *validateUsrParams[] = {hex_uuid, hex_installer_type, hex_language,
        hex_features, hex_install_tuning, hex_update_notify, hex_db_version,
        hex_os, hex_no_of_proc, hex_proc_arch, hex_chk_cnt, hex_proc_type,
        hex_ram_gb, hex_disk_gb, hex_shared_mem_mb, hex_userid, hex_passwd,
        hex_server_util, hex_work_profile};

    char *resultStr;
    ArrayOf_USCORExsd_USCOREstring validateUsrParamList;
    //ArrayOf_USCORExsd_USCOREstring createNewUsrParamList;
    
    //parameters list for createNewUser
    //createNewUsrParamList.__ptr = createNewUserParams;
    //createNewUsrParamList.__size = 18;

	//parameters list for validateUsrParams
	validateUsrParamList.__ptr = validateUsrParams;
	validateUsrParamList.__size = 19;

    //result = authBody.createNewUser(&createNewUsrParamList, resultStr);

    result = authBody.validateUser(&validateUsrParamList, resultStr);

    switch(resultStr[0] - '0')
    {
    case 0: //SUCCESS
        result = 0;
        break;
    case 1: //INVALID_EMAIL
        result = 1;
        break;
    case 2: //UNVALIDATE_EMAIL
        result = 2;
        break;
    case 4: //OTHER_ERROR
        result = 4;
        break;
    default: //Unexpected
        result = -1;
    }

    free(hex_uuid);
    free(hex_installer_type);
    free(hex_language);
    free(hex_features);
    free(hex_install_tuning);
    free(hex_update_notify);
    free(hex_db_version);
    free(hex_os);
    free(hex_no_of_proc);
    free(hex_proc_arch);
    free(hex_chk_cnt);
    free(hex_proc_type);
    free(hex_ram_gb);
    free(hex_disk_gb);
    free(hex_shared_mem_mb);
    free(hex_userid);
    free(hex_passwd);
    free(hex_server_util);

    free(l_language);
    free(l_ram_gb);
    free(l_ram_mb);

    return result;
}

ResultData *WSUtils::GetDynatuneInfo(
                             AuthenticationServiceSoapBindingProxy& authBody,
                             wxString &uuid, wxString &server_utilization,
                             wxString &workload_profile, int &len) {
    char *ram_gb = WSUtils::GetTotalMemoryInGB();
    char *ram_mb = WSUtils::GetTotalMemoryInMB();
    char *hexedUUID
             = WSUtils::ConvertToHexString(
                          (const char *)uuid.mb_str(wxConvUTF8));
    char *hexedSU
             = WSUtils::ConvertToHexString(
                          (const char *)server_utilization.mb_str(wxConvUTF8));
    char *hexedWP
             = WSUtils::ConvertToHexString(
                          (const char *)workload_profile.mb_str(wxConvUTF8));
    char *hexedRAMGB = WSUtils::ConvertToHexString(ram_gb);
    char *hexedRAMMB = WSUtils::ConvertToHexString(ram_mb);
    char *dynatuneParams[]
             = {hexedUUID, hexedSU, hexedRAMMB, hexedRAMGB, hexedWP};

    //contains returned parameters
    ResultData *dynatune
             = (ResultData *)malloc(sizeof(ResultData) * DYNATUNE_CONFIG_LEN);
    ArrayOf_USCORExsd_USCOREstring dynaParamList;
    ns2__getDynaTuneInfoResponse dynatuneResponse;
    ArrayOfArrayOf_USCORExsd_USCOREstring * dynatuneArrayofArray;

    //parameters list for dynatune
    dynaParamList.__ptr = dynatuneParams;
    dynaParamList.__size = 5;

    if(authBody.getDynaTuneInfo(&dynaParamList, dynatuneResponse) == SOAP_OK) {
        dynatuneArrayofArray = dynatuneResponse._getDynaTuneInfoReturn;
        if(dynatuneArrayofArray->__size != DYNATUNE_CONFIG_LEN) {
            ::wxLogError(
                    wxT("Dynatune variables list has been modified.\n"\
                        "Please visit http://forums.enterprisedb.com "\
                        "for further assistance"));
            free(dynatune);
            dynatune = NULL;
	}
        else
        {
            len = dynatuneArrayofArray->__size;
#ifdef ___CREATE_LOG_
            wxTextFile aLog(wxT("edb_webservice.log"));
            aLog.Create();
#endif //___CREATE_LOG_
            for(int index=0; index < dynatuneArrayofArray->__size; index++)
            {
#ifdef ___CREATE_LOG_
                wxString buffer;
                buffer
                   = wxString::FromAscii(
                                   dynatuneArrayofArray->__ptr[index].__ptr[0]);
                buffer += wxT(" = ");
                buffer
                   += wxString::FromAscii(
                                   dynatuneArrayofArray->__ptr[index].__ptr[1]);
                aLog.AddLine(buffer);
#endif //___CREATE_LOG_
                dynatune[index].prop = HexStrToStr(dynatuneArrayofArray->__ptr[index].__ptr[0]);
                dynatune[index].value = HexStrToStr(dynatuneArrayofArray->__ptr[index].__ptr[1]);
            }
#ifdef ___CREATE_LOG_
            aLog.Write();
            aLog.Close();
#endif //___CREATE_LOG_
        }
	}
	else {
		authBody.soap_stream_fault(std::cerr);
		free(dynatune);
		dynatune = NULL;
	}

	//free resources
	free(ram_gb);
	free(ram_mb);
	free(hexedUUID);
	free(hexedSU);
	free(hexedWP);
	free(hexedRAMGB);
	free(hexedRAMMB);

	return dynatune;
}


RSData::RSData()
{
    prop = NULL;
    value = NULL;
}

RSData::~RSData()
{
    if(prop)
        delete prop;
    if(value)
        delete value;
}
