/////////////////////////////////////////////////////////////////////////////
// Name:        WelcomePage.cpp
// Purpose:     Wizard page for greeting the user and detecting any 
//				server installation
// Author(s):   Atul, Ashesh
// Created:     2008-07-08
// RCS-ID:      $Id: WelcomePage.cpp,v 1.11 2008/08/21 06:43:12 atuld Exp $
// Copyright:   (c) EnterpriseDB
/////////////////////////////////////////////////////////////////////////////

#include "TuningTool.h"

// wxWindows headers
#include <wx/wx.h>
#include <wx/busyinfo.h>
#include <wx/utils.h>
#include <wx/wizard.h>
#include <wx/file.h>
#include <wx/gbsizer.h>

#ifdef __WXMSW__
#include <wx/msw/registry.h>
#else
#include <wx/wfstream.h>
#include <wx/fileconf.h>
#endif

// Application headers
#include "WelcomePage.h"
#include "ProxyDialog.h"
#include "Server.h"
#include "images/Tuning.xpm"

#define REGISTRY_FILE wxT("/etc/postgres-reg.ini")

const int BTN_PROXIES=1002;
const int CBX_INSTALLATION=1003;

BEGIN_EVENT_TABLE(WelcomePage, wxWizardPageSimple)
    EVT_BUTTON(BTN_PROXIES,            WelcomePage::OnProxies)
    EVT_WIZARD_PAGE_CHANGING(wxID_ANY, WelcomePage::OnWizardPageChanging)
END_EVENT_TABLE()

WelcomePage::WelcomePage(wxWizard *parent)
    : wxWizardPageSimple(parent)
{
    wxBoxSizer *mainSizer = new wxBoxSizer(wxVERTICAL);
    
    m_proxies = NULL;
    m_installation = NULL;
    m_fileSelector = NULL;
    m_installationDetected = false;
    
    SetBackgroundColour(*wxWHITE);

    mainSizer->Add(0, 10);

    wxStaticText *st = new wxStaticText(this, wxID_ANY, 
#ifdef WIN32
        wxT("Welcome to the EnterpriseDB Tuning Wizard"));
#else
        wxT("********************************************************************"));
#endif
    wxFont biggerFont = st->GetFont();
#ifdef WIN32
    biggerFont.SetPointSize(13);
#else
    biggerFont.SetPointSize(11);
#endif
    biggerFont.SetWeight(wxFONTWEIGHT_BOLD);
    st->SetFont(biggerFont);
    st->Wrap(400);
#ifndef WIN32
    mainSizer->Add(st, 0, wxALL | wxALIGN_LEFT| wxFIXED_MINSIZE, 10);
    st->SetLabel(wxT("Welcome to the EnterpriseDB Tuning Wizard"));
#else
    mainSizer->Add(st, 0, wxALL | wxALIGN_CENTER | wxFIXED_MINSIZE, 10);
#endif

    st = new wxStaticText(this, wxID_ANY, 
        wxT("This wizard will perform basic tuning of your PostgreSQL "\
            "installation based on the options you specify and your hardware "\
            "specifications.\n\nNote that systems under very high load or "\
            "used for benchmarking may also require further manual tuning for "\
            "the specific workload to obtain maximum performance."));
    st->Wrap(400);
    mainSizer->Add(st, 0, wxALL | wxFIXED_MINSIZE, 5);
#ifdef WIN32
    mainSizer->SetItemMinSize(st, 400, 70);
#else
    mainSizer->SetItemMinSize(st, 400, 110);
#endif

    m_installation = new wxComboBox(this, CBX_INSTALLATION, 
        wxT(""), wxDefaultPosition, wxDefaultSize, 0, NULL, 
        wxCB_DROPDOWN | wxCB_READONLY);

    // hide the combobox until we are sure that an installation is detected
    m_installation->Show(false);

    m_installationDetected = FindPgServers() || FindEdbServers();
    if (m_installationDetected)
    {
        st = new wxStaticText(this, wxID_ANY, 
            wxT("To begin, please select the installation that you would "\
                "like to tune from the list below."));
        st->Wrap(400);
        mainSizer->Add(st, 0, wxALL| wxFIXED_MINSIZE, 5);
#ifdef WIN32
        mainSizer->SetItemMinSize(st, 400, 30);
#else
        mainSizer->SetItemMinSize(st, 400, 35);
#endif

        mainSizer->Add(m_installation, 0, wxALL | wxALIGN_CENTER, 5);
        // un-hide the combobox
        m_installation->Show(true);
#ifdef WIN32
        mainSizer->Add(5, 80);
#else
        mainSizer->Add(5, 45);
#endif
    }
    else
    {
        st = new wxStaticText(this, wxID_ANY, 
            wxT("We could not find a PostgreSQL or EnterpriseDB installation. "\
                "Please select the postgresql.conf for the tuning."));
        st->Wrap(400);
        mainSizer->Add(st, 0, wxALL| wxFIXED_MINSIZE, 5);
#ifdef WIN32
        mainSizer->SetItemMinSize(st, 400, 30);
#else
        mainSizer->SetItemMinSize(st, 400, 35);
#endif

        wxBoxSizer *rowSizer = new wxBoxSizer( wxHORIZONTAL );
        m_fileSelector = new FileSelectorCombo(this, wxID_ANY, wxEmptyString,
            wxPoint(0,0), wxSize(300, 25), (long)0);
        rowSizer->Add(m_fileSelector, 0, wxALL, 4);
        mainSizer->Add(rowSizer, 0, wxEXPAND|wxALL, 5 );
#ifdef WIN32
        mainSizer->Add(5, 80);
#else
        mainSizer->Add(5, 30);
#endif
    }

    m_proxies = new wxButton(this, BTN_PROXIES, wxT("Proxy server"));
#ifdef WIN32
    mainSizer->Add(m_proxies, 0, wxALL | wxALIGN_LEFT, 15);
#else
    mainSizer->Add(m_proxies, 0, wxALL | wxALIGN_LEFT, 5);
#endif

    mainSizer->AddStretchSpacer();

    wxGridBagSizer *imageSizer = new wxGridBagSizer();

    imageSizer->SetMinSize(20, 5);
#if wxUSE_STATBMP
    wxBoxSizer *bmpSizer = new wxBoxSizer(wxVERTICAL);
    imageSizer->Add(
        bmpSizer, wxGBPosition(0,32), wxGBSpan(3,1), wxALIGN_RIGHT
        );

    // Create and run the wizard
    wxBitmap bitmap = wxBitmap(Tuning_xpm);

    if ( bitmap.Ok() )
    {
        wxStaticBitmap *statbmp = new wxStaticBitmap(this, wxID_ANY, bitmap);
        bmpSizer->Add(statbmp, 0, wxALL, 0);
    }
#endif
    mainSizer->Add(imageSizer);

    SetSizer(mainSizer);
    mainSizer->Fit(this);
}

bool WelcomePage::FindPgServers()
{
    bool success = false;
    bool flag    = false;
    long cookie  = 0;
    long port    = 0;
    wxString temp;
#ifdef __WXMSW__
    // Add local servers.
    wxRegKey *rootKey = new wxRegKey(wxT("HKEY_LOCAL_MACHINE\\Software\\PostgreSQL\\Services"));

    if (rootKey->Exists())
    {
        wxString svcName, temp;
        long cookie = 0;
        long port = 0;
        bool flag = false;

        flag = rootKey->GetFirstKey(svcName, cookie);

        while (flag != false)
        {
            wxString keyName, guid;
            Server *data = new Server();
            data->m_serverType = SVR_POSTGRESQL;

            // Get the service data
            data->m_serviceId = svcName;
            keyName.Printf(wxT("HKEY_LOCAL_MACHINE\\Software\\PostgreSQL\\Services\\%s"), svcName.c_str());
            wxRegKey *svcKey = new wxRegKey(keyName);
            svcKey->QueryValue(wxT("Display Name"), data->m_description);
            svcKey->QueryValue(wxT("Port"), &data->m_port);
            svcKey->QueryValue(wxT("Data Directory"), data->m_dataDirectory);
            svcKey->QueryValue(wxT("Database Superuser"), data->m_superuserName);
            svcKey->QueryValue(wxT("Service Account"), data->m_serviceAccount);
            svcKey->QueryValue(wxT("Encoding"), data->m_encoding);
            svcKey->QueryValue(wxT("Locale"), data->m_locale);

            // Get the version number from installation record
            svcKey->QueryValue(wxT("Product Code"), guid);
            keyName.Printf(wxT("HKEY_LOCAL_MACHINE\\Software\\PostgreSQL\\Installations\\%s"), guid.c_str());
            wxRegKey *instKey = new wxRegKey(keyName);
            instKey->QueryValue(wxT("Version"), data->m_serverVersion);
            data->m_serverVersion.BeforeFirst('.').ToLong(&data->m_majorVer);
            data->m_serverVersion.AfterFirst('.').ToLong(&data->m_minorVer);
            instKey->QueryValue(wxT("Base Directory"), data->m_installationPath);

            // Build the user description
            temp.Printf(
                  wxT("%s on port %d"),
                  data->m_description.c_str(), data->m_port);

            // Add the item
            if(IsValidPGServer(data))
            {
                m_installation->Append(temp, data);
                success = true;
            }
            else
            {
                delete(data);
                data = NULL;
            }
            // Get the next one...
            flag = rootKey->GetNextKey(svcName, cookie);
        }
    }
#else
    if (wxFile::Exists(REGISTRY_FILE))
    {
        wxString version, locale;
        long cookie;
        
        wxFileStream fst(REGISTRY_FILE);
        wxFileConfig *cnf = new wxFileConfig(fst);
        
        cnf->SetPath(wxT("/PostgreSQL"));
        flag = cnf->GetFirstGroup(version, cookie);
        while (flag)
        {
            // If there is no Version entry, this is probably an uninstalled server
            if (cnf->Read(version + wxT("/Version"), wxEmptyString) != wxEmptyString)
            {
                Server *data = new Server();
                data->m_serverType = SVR_POSTGRESQL;
                
                // Server version
                data->m_serverVersion = version;
                data->m_serverVersion.BeforeFirst('.').ToLong(&data->m_majorVer);
                data->m_serverVersion.AfterFirst('.').ToLong(&data->m_minorVer);
            
                // And the rest of the data
                data->m_description = cnf->Read(version + wxT("/Description"), _("Unknown server"));
                data->m_port = cnf->Read(version + wxT("/Port"), 0L);
                data->m_dataDirectory = cnf->Read(version + wxT("/DataDirectory"), wxEmptyString);
                data->m_installationPath = cnf->Read(version + wxT("/InstallationDirectory"), wxEmptyString);
                data->m_superuserName = cnf->Read(version + wxT("/Superuser"), wxEmptyString);
                data->m_serviceAccount = cnf->Read(version + wxT("/Superuser"), wxEmptyString); // We use the Superuser entry for both on *nix
                
                // Separate the locale and encoding if possible
                locale = cnf->Read(version + wxT("/Locale"), wxEmptyString);
                
                if (locale.Find('.') == wxNOT_FOUND)
                {
                    data->m_locale = locale;
                }
                else
                {
                    data->m_locale = locale.BeforeFirst('.');
                    data->m_encoding = locale.AfterFirst('.');
                }
                
                // Build the user description
                temp.Printf(_("%s on port %d"), data->m_description.c_str(), data->m_port);
                
                // Add the item
                m_installation->Append(temp, data);
                success = true;
            }
            
            flag = cnf->GetNextGroup(version, cookie);
        }
        
        delete cnf;
    }
#endif
    return success;
}

bool WelcomePage::IsValidPGServer(Server *server)
{
    if(server && server->m_majorVer == 8)
        if(server->m_minorVer == 2 || server->m_minorVer == 3)
            return true;
    return false;
}

// Note: The following function is currently the same as FindPgServers() just
//       using different registry keys until the EDB installer starts writing 
//       it's install data to the registry.
bool WelcomePage::FindEdbServers()
{
    bool success = false;
    long cookie = 0;
    long port = 0;
    bool flag = false;
    wxString temp;
#ifdef __WXMSW__
    // Add local servers.
    wxRegKey *rootKey = new wxRegKey(wxT("HKEY_LOCAL_MACHINE\\Software\\EnterpriseDB\\Services"));

    if (rootKey->Exists())
    {
        wxString svcName;

        flag = rootKey->GetFirstKey(svcName, cookie);

        while (flag != false)
        {
            wxString keyName, guid;
            Server *data = new Server();
            data->m_serverType = SVR_ENTERPRISEDB;

            // Get the service data
            keyName.Printf(wxT("HKEY_LOCAL_MACHINE\\Software\\EnterpriseDB\\Services\\%s"), svcName.c_str());
            wxRegKey *svcKey = new wxRegKey(keyName);
            svcKey->QueryValue(wxT("Display Name"), data->m_description);
            svcKey->QueryValue(wxT("Port"), &data->m_port);
            svcKey->QueryValue(wxT("Data Directory"), data->m_dataDirectory);
            svcKey->QueryValue(wxT("Database Superuser"), data->m_superuserName);
            svcKey->QueryValue(wxT("Service Account"), data->m_serviceAccount);
            svcKey->QueryValue(wxT("Encoding"), data->m_encoding);
            svcKey->QueryValue(wxT("Locale"), data->m_locale);

            // Get the version number from installation record
            svcKey->QueryValue(wxT("Product Code"), guid);
            keyName.Printf(
               wxT("HKEY_LOCAL_MACHINE\\Software\\EnterpriseDB\\Installations"\
                   "\\%s"), guid.c_str());
            wxRegKey *instKey = new wxRegKey(keyName);
            instKey->QueryValue(wxT("Version"), data->m_serverVersion);
            data->m_serverVersion.BeforeFirst('.').ToLong(&data->m_majorVer);
            data->m_serverVersion.AfterFirst('.').ToLong(&data->m_minorVer);
            instKey->QueryValue(wxT("Base Directory"), data->m_installationPath);

            // Build the user description
            temp.Printf(
                        wxT("%s on port %d"),
                        data->m_description.c_str(),
                        data->m_port);

            // Add the item
            if(IsValidPGServer(data))
            {
                m_installation->Append(temp, data);
                success = true;
            }
            else
            {
                delete(data);
                data = NULL;
            }
            // Get the next one...
            flag = rootKey->GetNextKey(svcName, cookie);
        }
    }
#else
    if (wxFile::Exists(REGISTRY_FILE))
    {
        wxString version, locale;
        long cookie;
        
        wxFileStream fst(REGISTRY_FILE);
        wxFileConfig *cnf = new wxFileConfig(fst);
        
        cnf->SetPath(wxT("/EnterpriseDB"));
        flag = cnf->GetFirstGroup(version, cookie);
        while (flag)
        {
            // If there is no Version entry, this is probably an uninstalled server
            if (cnf->Read(version + wxT("/Version"), wxEmptyString) != wxEmptyString)
            {
                Server *data = new Server();
                data->m_serverType = SVR_ENTERPRISEDB;
                
                // Server version
                data->m_serverVersion = version;
                data->m_serverVersion.BeforeFirst('.').ToLong(&data->m_majorVer);
                data->m_serverVersion.AfterFirst('.').ToLong(&data->m_minorVer);
                
                // And the rest of the data
                data->m_description = cnf->Read(version + wxT("/Description"), _("Unknown server"));
                data->m_port = cnf->Read(version + wxT("/Port"), 0L);
                data->m_dataDirectory = cnf->Read(version + wxT("/DataDirectory"), wxEmptyString);
                data->m_installationPath = cnf->Read(version + wxT("/InstallationDirectory"), wxEmptyString);
                data->m_superuserName = cnf->Read(version + wxT("/Superuser"), wxEmptyString);
                data->m_serviceAccount = cnf->Read(version + wxT("/Superuser"), wxEmptyString); // We use the Superuser entry for both on *nix
                
                // Separate the locale and encoding if possible
                locale = cnf->Read(version + wxT("/Locale"), wxEmptyString);
                
                if (locale.Find('.') == wxNOT_FOUND)
                {
                    data->m_locale = locale;
                }
                else
                {
                    data->m_locale = locale.BeforeFirst('.');
                    data->m_encoding = locale.AfterFirst('.');
                }
                
                // Build the user description
                temp.Printf(_("%s on port %d"), data->m_description.c_str(), data->m_port);
                
                // Add the item
                m_installation->Append(temp, data);
                success = true;
            }
            
            flag = cnf->GetNextGroup(version, cookie);
        }
        
        delete cnf;
    } 
#endif
    return success;
}

void WelcomePage::OnWizardPageChanging(wxWizardEvent& event)
{
    if (m_installationDetected)
    {
        if (m_installation->GetSelection() == wxNOT_FOUND)
        {
            wxLogError(
               wxT("You must select the installation to tune before you "\
                   "continue."));
            event.Veto();
            return;
        }
    }
    else 
    {
        if (m_fileSelector->GetFilename().IsEmpty())
        {
            wxLogError(
               wxT("You must select the postgresql.conf to tune before you "\
                   "continue."));
            event.Veto();
            return;
        }
        if(!wxFile::Exists(m_fileSelector->GetPath()))
        {
            wxString errorMsg;
            errorMsg.Printf(
                         wxT("File \'%s\' not found"),
                         m_fileSelector->GetPath().c_str());
            wxLogError(wxT("File not found."));
            event.Veto();
            return;
        }
        m_EmptyServer.m_dataDirectory
                        = m_fileSelector->GetDirectory() + wxT("/");
    }
}

void WelcomePage::OnProxies(wxCommandEvent& WXUNUSED(event))
{
    ProxyDialog *pd = new ProxyDialog(this, wxT("Proxy server configuration"));
    pd->ShowModal();
    delete pd;
}
