/////////////////////////////////////////////////////////////////////////////
// Name:        Wizard.cpp
// Purpose:     The main wizard class
// Author(s):   Atul, Ashesh
// Created:     2008-07-08
// RCS-ID:      $Id: Wizard.cpp,v 1.5 2008/08/18 14:18:35 asheshv Exp $
// Copyright:   (c) EnterpriseDB
/////////////////////////////////////////////////////////////////////////////

#include "TuningTool.h"

// wxWindows headers
#include <wx/wx.h>
#include <wx/wizard.h>

// Application headers
#include "Wizard.h"
#include "WelcomePage.h"
#include "SysInfoPage.h"
#include "CompletionPage.h"
#include "ThankYouPage.h"

Wizard::Wizard(wxFrame *frame, wxBitmap bitmap)
    : wxWizard(frame, wxID_ANY, wxT("EnterpriseDB Tuning Wizard"), bitmap)
{
	m_bitmap = bitmap;
	SetBackgroundColour(*wxWHITE);

    // Add the pages
    m_page1 = new WelcomePage(this);
    m_page2 = new SysInfoPage(this);
    m_page3 = new CompletionPage(this);
    m_page4 = new ThankYouPage(this);

    // Join 'em all up
    wxWizardPageSimple::Chain(m_page1, m_page2);
    wxWizardPageSimple::Chain(m_page2, m_page3);

	// The last page should have the back button disabled, 
	// so don't chain it, just add it as a next page
	m_page3->SetNext(m_page4);

    // Allow the wizard to size itself around the pages
    /*
     * Since a problem is in correspondence of Japanese font size, it is made fixed 
     * size. However, in the future version of wxWidgets, an original code is desirable. 
     * GetPageAreaSizer()->Add(m_page1);
     */
    GetPageAreaSizer()->Add(420, 0);

    SetBorder(0);
}

Server* Wizard::GetServerConfiguration(int& svr_utilization, int& workload_profile)
{
    svr_utilization = ((SysInfoPage *)m_page2)->GetServerUtillization();
    workload_profile = ((SysInfoPage *)m_page2)->GetDBMSUsage();
    return ((WelcomePage *)m_page1)->GetSelectedServer();
}
